#define _CRT_SECURE_NO_WARNINGS
//
// Copyright (c) 2004 - InfoMus Lab - DIST - University of Genova
//
// InfoMus Lab (Laboratorio di Informatica Musicale)
// DIST - University of Genova
//
// http://www.infomus.dist.unige.it
// news://infomus.dist.unige.it
// mailto:staff@infomus.dist.unige.it
//
// Developer: Gualtiero Volpe
// mailto:volpe@infomus.dist.unige.it
//
// Developer: Trey Harrison
// mailto:trey@treyharrison.com
//
// Last modified: Oct. 26 2006
//

#include "PluginBase.h"
#include <stdio.h>
#include <memory.h>
#include <algorithm>

namespace ffgl
{

// Buffer used by the default implementation of getParameterDisplay
static char s_DisplayValue[ 16 ];

////////////////////////////////////////////////////////
// PluginBase constructor and destructor
////////////////////////////////////////////////////////

PluginBase::PluginBase() :
	PluginManager(),
	bpm( 120.0f ),
	barPhase( 0.0f )
{
}

PluginBase::~PluginBase()
{
}

////////////////////////////////////////////////////////
// Default implementation of PluginBase methods
////////////////////////////////////////////////////////

char* PluginBase::GetParameterDisplay( unsigned int index )
{
	ParameterType pType = m_pPlugin->GetParamType( index );
	if( pType != ParameterType::FF_FAIL )
	{
		if( pType == ParameterType::FF_TYPE_TEXT )
		{
			return m_pPlugin->GetTextParameter( index );
		}
		else
		{
			std::string stringValue = std::to_string(m_pPlugin->GetFloatParameter( index ));
			memset(s_DisplayValue, 0, sizeof(s_DisplayValue));
			memcpy(s_DisplayValue, stringValue.c_str(), std::min(sizeof(s_DisplayValue), stringValue.length()));
			return s_DisplayValue;
		}
	}

	return nullptr;
}

FFResult PluginBase::SetFloatParameter( unsigned int index, float value )
{
	return FF_FAIL;
}

FFResult PluginBase::SetTextParameter( unsigned int index, const char* value )
{
	return FF_FAIL;
}

float PluginBase::GetFloatParameter( unsigned int index )
{
	return 0.0;
}

char* PluginBase::GetTextParameter( unsigned int index )
{
	return (char*)FF_FAIL;
}

InputStatus PluginBase::GetInputStatus( unsigned int index )
{
	if( index >= GetMaxInputs() )
		return InputStatus::FF_FAIL;

	return InputStatus::FF_INPUT_INUSE;
}

void PluginBase::SetBeatInfo( float bpm, float barPhase )
{
	this->bpm = bpm;
	this->barPhase = barPhase;
}

void PluginBase::SetHostInfo(const char * hostname, const char * version)
{
	hostInfos.name = hostname;
	hostInfos.version = version;
}

void PluginBase::SetSampleRate(unsigned int _sampleRate)
{
	sampleRate = _sampleRate;
}

}//End namespace ffgl
