#include "Audio.h"
#include <cmath>

Audio::Audio()
{
}

Audio::~Audio()
{
}

void Audio::update( std::vector< float > _fft )
{
	//TODO: find out if the "FFT" data is actually results of FFT or raw audio data

	//TODO: this seems to be the third copy of the FFT data:
	// - 1: original plugin manager buffer PluginManager::elements[]
	// - 2: copied to ParamFFT::fftData[]
	// - 3: local copy Audio::fft[]
	// Remove redundant copies?

	// Copy the FFT data
	fft = _fft;

	int fftCount = (int) fft.size(); // Save calling .size() over and over

	// Determine indicies for differentiation between bass, medium and high frequencies
	int bassSep  = (int)( ceil( fftCount / 3.0 ) );        // 2019-07-10 ffgl@subpixels.com - slighty bias bass over med (in case of few fft bins)
	int highSep  = (int)( floor( fftCount * 2.0 / 3.0 ) ); // 2019-07-10 ffgl@subpixels.com - slighty bias high over med (in case of few fft bins)

	// Overall volume level
	float currentVolStore = 0;

	// Bass frequencies
	float bassStore = 0;
	float bassCount = 0;

	// Medium frequencies
	float medStore  = 0;
	float medCount  = 0;

	// High frequencies
	float highStore = 0;
	float highCount = 0;

	for( int i = 0; i < fftCount; i++ )
	{
		float bin = fft[ i ] * fft[ i ] * gain;

		if( i < bassSep )
		{
			bassStore += bin;
			bassCount++;
		}
		else if( i < highSep )
		{
			medStore += bin;
			medCount++;
		}
		else
		{
			highStore += bin;
			highCount++;
		}

		currentVolStore += bin;
	}

	// Be wary of divide by zero in calculations
	vol.update(fftCount == 0 ? 0 : sqrt( currentVolStore / fftCount ) );
	bass.update( bassCount == 0 ? 0 : sqrt( bassStore / bassCount ) );
	med.update( medCount == 0 ? 0 : sqrt( medStore / medCount ) );
	high.update( highCount == 0 ? 0 : sqrt( highStore / highCount ) );
}

float Audio::getVolume()
{
	return vol.getValue();
}

float Audio::getBass()
{
	return bass.getValue();
}

float Audio::getMed()
{
	return med.getValue();
}

float Audio::getHigh()
{
	return high.getValue();
}

float Audio::getVolumeFromTo( int fromFreq, int toFreq )
{
	//TODO: not yet implemented
	return 0.0f;
}

float Audio::toDb( float rms )
{
	float db = 20 * std::log10( rms );
	db       = utils::map( db, -96, 0, 0, 1 );
	if( std::isinf( db ) )
		db = 0;
	return db;
}

void Audio::setSmoothness( float smoothness )
{
	vol.setSmoothness( smoothness );
	bass.setSmoothness( smoothness );
	med.setSmoothness( smoothness );
	high.setSmoothness( smoothness );
}

void Audio::setSampleRate( int _sampleRate )
{
	sampleRate  = _sampleRate;
	freqMax     = (float) sampleRate / 2.f;
	freqBinStep = freqMax / fft.size();
}

void Audio::setGain( float _gain )
{
	gain = pow( 10.f, _gain / 20.f );
}
